"""Build optional cython modules."""

import os
from distutils.command.build_ext import build_ext
from os.path import join
from typing import Any

try:
    from setuptools import Extension
except ImportError:
    from distutils.core import Extension

ulid_module = Extension(
    "fnv_hash_fast._fnv_impl",
    [
        join("src", "fnv_hash_fast", "_fnv_impl.pyx"),
    ],
    language="c",
)


class BuildExt(build_ext):
    def build_extensions(self) -> None:
        try:
            super().build_extensions()
        except Exception:  # nosec
            pass


def build(setup_kwargs: Any) -> None:
    if os.environ.get("SKIP_CYTHON", False):
        return
    try:
        from Cython.Build import cythonize

        setup_kwargs.update(
            dict(
                ext_modules=cythonize(
                    [
                        ulid_module,
                    ],
                    compiler_directives={"language_level": "3"},  # Python 3
                ),
                cmdclass=dict(build_ext=BuildExt),
            )
        )
        setup_kwargs["exclude_package_data"] = {
            pkg: ["*.c"] for pkg in setup_kwargs["packages"]
        }
    except Exception:
        if os.environ.get("REQUIRE_CYTHON"):
            raise
        pass



from functools import partial
from typing import TYPE_CHECKING, Callable

from fnvhash import FNV1_32_INIT, FNV_32_PRIME, fnva

_FNV_SIZE = 2**32

if TYPE_CHECKING:
    fnv1a_32: Callable[[bytes], int]

fnv1a_32 = partial(
    fnva, hval_init=FNV1_32_INIT, fnv_prime=FNV_32_PRIME, fnv_size=_FNV_SIZE
)

try:
    from ._fnv_impl import (  # type: ignore[no-redef] # noqa: F811 F401
        _fnv1a_32 as fnv1a_32,
    )
except ImportError:
    pass
